#version 330

in vec4 vCol;
in vec2 TexCoord;
in mat3 TBN;
in vec3 FragPos;
in vec4 FragPosLightSpace;

layout (location = 0) out vec4 FragColor;
layout (location = 1) out vec4 BrightColor;

struct DirectionalLight 
{
	vec3 colour;
	float ambientIntensity;
	vec3 direction;
	float diffuseIntensity;
};

struct Material
{
	float specularIntensity;
	float shininess;
};

uniform sampler2D diffuseMap;
uniform sampler2D shadowMap;
uniform sampler2D specularMap;
uniform sampler2D normalMap;


uniform DirectionalLight directionalLight;
uniform Material material;

uniform vec3 eyePosition;
uniform bool debugNormals = false;

float ShadowCalculation(vec4 fragPosLightSpace)
{
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    projCoords = projCoords * 0.5 + 0.5; 
	float closestDepth = texture(shadowMap, projCoords.xy).r;   
	float currentDepth = projCoords.z;
	float shadow = currentDepth > closestDepth  ? 1.0 : 0.0;  

	return shadow;
}

void main()
{		
	
	vec3 normal =  texture(normalMap, TexCoord).rgb * 2 - 1;
	normal = normalize(TBN * normal); 


	vec4 ambientColour = vec4(directionalLight.colour, 1.0f) * directionalLight.ambientIntensity;
	
	float diffuseFactor = max(dot(normalize(normal), normalize(directionalLight.direction)), 0.0f);
	vec4 diffuseColour = vec4(directionalLight.colour, 1.0f) * directionalLight.diffuseIntensity * diffuseFactor;
	
	vec4 specularColour = vec4(0, 0, 0, 1);
	
	if(diffuseFactor > 0.0f)
	{
		vec3 fragToEye = normalize(eyePosition - FragPos);
		vec3 reflectedVertex = normalize(reflect(directionalLight.direction, normalize(normal)));
		
		float specularFactor = dot(fragToEye, reflectedVertex);
		if(specularFactor > 0.0f)
		{
			specularFactor = pow(specularFactor, material.shininess);
			specularFactor *= texture(specularMap, TexCoord).r;
			specularColour = vec4(directionalLight.colour * material.specularIntensity * specularFactor, 1.0f);
		}
	}
	float shadow = 0.1f + ShadowCalculation(FragPosLightSpace);
	FragColor = texture(diffuseMap, TexCoord) * (ambientColour + diffuseColour + specularColour);
	FragColor.a = 1;

	if (debugNormals) {
		FragColor.rgb = vec3(normal);
	}
	
	// for bloom effect check
	float brightness = dot(FragColor.rgb, vec3(0.2126, 0.7152, 0.0722));
	BrightColor = vec4(mix(vec3(0), FragColor.rgb*2, clamp(brightness*3-2.9, 0, 1)), 1);
}