#version 330

in vec4 vCol;
in vec2 TexCoord;
in mat3 TBN;
in vec3 FragPos;
in vec4 FragPosLightSpace;

layout (location = 0) out vec4 FragColor;
layout (location = 1) out vec4 BrightColor;

struct DirectionalLight 
{
	vec3 colour;
	float ambientIntensity;
	vec3 direction;
	float diffuseIntensity;
};

struct Material
{
	float specularIntensity;
	float shininess;
	float reflectivity;
};

uniform sampler2D diffuseMap;
uniform sampler2D shadowMap;
uniform sampler2D specularMap;
uniform sampler2D normalMap;
uniform samplerCube environmentMap;

uniform DirectionalLight directionalLight;
uniform Material material;

uniform vec3 eyePosition;
uniform bool debugNormals = false;
uniform bool smoothShadows = true;

float isFragmentInShadow(vec4 fragPosLightSpace, vec3 normal, vec3 lightDir)
{
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    projCoords = projCoords * 0.5 + 0.5; 
	
	// no shadows far away from light
    if(projCoords.z > 1.0)
        return 0.0;
		
	// bias depending on angle
	float bias = max(0.01 * (1.0 - dot(normal, lightDir)), 0.005); 

	// compare fragment depths in lightspace
	if (smoothShadows) {
		vec2 texelSize = 1.0 / textureSize(shadowMap, 0);
		float shadow = 0.0;
		for (int i = -1; i <= 1; i++) {
			for (int j = -1; j <= 1; j++) {
				float closestDepth = texture(shadowMap, projCoords.xy + vec2(i, j) * texelSize).r;   
				float currentDepth = projCoords.z;
				shadow += currentDepth - bias > closestDepth  ? 1.0 : 0.0;  
			}
		}
		return shadow / 9.0;
	} else {
		float closestDepth = texture(shadowMap, projCoords.xy).r;   
		float currentDepth = projCoords.z;
		float shadow = currentDepth - bias > closestDepth  ? 1.0 : 0.0;  
		return shadow;
	}
}

void main()
{		
	
	vec3 normal =  texture(normalMap, TexCoord).rgb * 2 - 1;
	normal = normalize(TBN * normal); 


	vec4 ambientColour = vec4(directionalLight.colour, 1.0f) * directionalLight.ambientIntensity;
	
	float diffuseFactor = max(dot(normalize(normal), normalize(directionalLight.direction)), 0.0f);
	vec4 diffuseColour = vec4(directionalLight.colour, 1.0f) * directionalLight.diffuseIntensity * diffuseFactor;
	
	vec4 specularColour = vec4(0, 0, 0, 1);
	
	if(diffuseFactor > 0.0f)
	{
		vec3 fragToEye = normalize(eyePosition - FragPos);
		vec3 reflectedVertex = normalize(reflect(directionalLight.direction, normalize(normal)));
		
		float specularFactor = dot(fragToEye, reflectedVertex);
		if(specularFactor > 0.0f)
		{
			specularFactor = pow(specularFactor, material.shininess);
			specularFactor *= texture(specularMap, TexCoord).r;
			specularColour = vec4(directionalLight.colour * material.specularIntensity * specularFactor, 1.0f);
		}
	}
	float directLight = 1.0 - isFragmentInShadow(FragPosLightSpace, normal, directionalLight.direction);
	FragColor = texture(diffuseMap, TexCoord) * (ambientColour + (directLight * (diffuseColour + specularColour)));
	FragColor.rgb *= 0.9;
	
	// environment reflection
	vec3 I = normalize(FragPos - eyePosition);
    vec3 R = reflect(I, normalize(normal));
    FragColor.rgb = FragColor.rgb * vec3(1-material.reflectivity) + texture(environmentMap, R).rgb * vec3(material.reflectivity);
	FragColor.a = 1;

	if (debugNormals) {
		FragColor.rgb = vec3(normal);
	}
	
	// for bloom effect check
	float brightness = dot(FragColor.rgb, vec3(0.2126, 0.7152, 0.0722));
	BrightColor = vec4(mix(vec3(0), FragColor.rgb*2, clamp(brightness*3-2.2, 0, 1)), 1);
}